"""A subclass of Attributes that allows for specifying optional and required attributes

"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass
from typing import Iterable


class AttributeTypesNotUniqueError(Exception):
    """Exception that is raised when the class is instantiated with two attributes of the same type."""

    def __init__(self, non_unique_types: Iterable[type]):
        super().__init__(
            f"Each Attribute type may only be provided once. This is not the case for {[o.__name__ for o in non_unique_types]}."
        )


@dataclass
class EachAttributeOnlyOnceEnforcer:
    """A base class for all Attribute classes that enforce optional and required attributes."""

    def _attribute_type_list(self) -> list[type]:
        attribute_type_list: list[type] = list()
        for dataclass_field in self.__dataclass_fields__.keys():
            value = getattr(self, dataclass_field)
            if value is None or isinstance(value, str):
                continue
            for attribute in value:
                attribute_type_list.append(attribute.__class__)
        return attribute_type_list

    def __post_init__(self) -> None:
        attribute_type_list = self._attribute_type_list()
        attribute_type_set = set(attribute_type_list)

        if len(attribute_type_list) != len(attribute_type_set):
            non_unique_types = [a for a in attribute_type_set if attribute_type_list.count(a) > 1]
            raise AttributeTypesNotUniqueError(non_unique_types)



"""Summarizing information of the object over the entire scenario"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass, field
from typing import Literal

from apischema.metadata import required
from uai_openlabel import NumberData, NumberType, VectorData, no_default


@dataclass
class Summary__Speed__Max(NumberData):
    """Maximum speed of the object over all frames in this file in (m/s)."""

    val: float = field(default_factory=lambda: no_default(field="Summary__Speed__Max.val"), metadata=required)
    """Maximum speed of the object in (m/s)."""

    name: Literal["summary/speed/max"] = field(default="summary/speed/max")
    """Is always 'summary/speed/max'"""


@dataclass
class Summary__Speed__Max__UStdDev(NumberData):
    """Uncertainty for `Summary__Speed__Max` as standard deviation around the parent indicated value."""

    val: float = field(default_factory=lambda: no_default(field="Summary__Speed__Max__UStdDev.val"), metadata=required)
    """Standard deviation around the parent indicated value. """

    name: Literal["summary/speed/max/ustddev"] = field(default="summary/speed/max/ustddev")
    """Is always 'summary/speed/max/ustddev'."""


@dataclass
class Summary__Speed__Min(NumberData):
    """Minimum speed of the object over all frames in this file in (m/s)."""

    val: float = field(default_factory=lambda: no_default(field="Summary__Speed__Min.val"), metadata=required)
    """Minimum speed of the object in (m/s)."""

    name: Literal["summary/speed/min"] = field(default="summary/speed/min")
    """Is always 'summary/speed/min'"""


@dataclass
class Summary__Speed__Min__UStdDev(NumberData):
    """Uncertainty for `Summary__Speed__Min` as standard deviation around the parent indicated value."""

    val: float = field(default_factory=lambda: no_default(field="Summary__Speed__Min__UStdDev.val"), metadata=required)
    """Standard deviation around the parent indicated value. """

    name: Literal["summary/speed/min/ustddev"] = field(default="summary/speed/min/ustddev")
    """Is always 'summary/speed/min/ustddev'."""


@dataclass
class Summary__Accel__Max(NumberData):
    """Maximum acceleration of the object over all frames in this file in (m/s^2)."""

    val: float = field(default_factory=lambda: no_default(field="Summary__Accel__Max.val"), metadata=required)
    """Maximum acceleration of the object in (m/s^2)."""

    name: Literal["summary/accel/max"] = field(default="summary/accel/max")
    """Is always 'summary/accel/max'"""

    type: Literal[NumberType.Value] = field(default=NumberType.Value)
    """Is always 'value'"""


@dataclass
class Summary__Accel__Max__UStdDev(NumberData):
    """Uncertainty for `Summary__Accel__Max` as standard deviation around the parent indicated value."""

    val: float = field(default_factory=lambda: no_default(field="Summary__Accel__Max__UStdDev.val"), metadata=required)
    """Standard deviation around the parent indicated value. """

    name: Literal["summary/accel/max/ustddev"] = field(default="summary/accel/max/ustddev")
    """Is always 'summary/accel/max/ustddev'."""


@dataclass
class Summary__Accel__Min(NumberData):
    """Minimum acceleration (strongest deceleration if negative) of the object over all frames in this file in (m/s^2)."""

    val: float = field(default_factory=lambda: no_default(field="Summary__Accel__Min.val"), metadata=required)
    """Minimum acceleration of the object in (m/s^2)."""

    name: Literal["summary/accel/min"] = field(default="summary/accel/min")
    """Is always 'summary/accel/min'"""

    type: Literal[NumberType.Value] = field(default=NumberType.Value)
    """Is always 'value'"""


@dataclass
class Summary__Accel__Min__UStdDev(NumberData):
    """Uncertainty for `Summary__Accel__Min` as standard deviation around the parent indicated value."""

    val: float = field(default_factory=lambda: no_default(field="Summary__Speed__Min__UStdDev.val"), metadata=required)
    """Standard deviation around the parent indicated value. """

    name: Literal["summary/accel/min/ustddev"] = field(default="summary/accel/min/ustddev")
    """Is always 'summary/accel/min/ustddev'."""


@dataclass
class Summary__SteeringWheelAngle__Max(NumberData):
    """Maximum steering wheel angle of the object over all frames in this file."""

    val: float = field(default_factory=lambda: no_default(field="Summary__SteeringWheelAngle__Max.val"), metadata=required)
    """Maximum steering wheel angle in radians, counter clockwise."""

    name: Literal["summary/steering_wheel_angle/max"] = field(default="summary/steering_wheel_angle/max")
    """Is always 'summary/steering_wheel_angle/max'"""


@dataclass
class Summary__SteeringWheelAngle__Max__UStdDev(NumberData):
    """Uncertainty for `Summary__SteeringWheelAngle__Max` as standard deviation around the parent indicated value."""

    val: float = field(
        default_factory=lambda: no_default(field="Summary__SteeringWheelAngle__Max__UStdDev.val"), metadata=required
    )
    """Standard deviation around the parent indicated value. """

    name: Literal["summary/steering_wheel_angle/max/ustddev"] = field(default="summary/steering_wheel_angle/max/ustddev")
    """Is always 'summary/steering_wheel_angle/max/ustddev'."""


@dataclass
class Summary__SteeringWheelAngle__Min(NumberData):
    """Minimum steering wheel angle of the object over all frames in this file."""

    val: float = field(default_factory=lambda: no_default(field="Summary__SteeringWheelAngle__Min.val"), metadata=required)
    """Minimum steering wheel angle in radians, counter clockwise."""

    name: Literal["summary/steering_wheel_angle/min"] = field(default="summary/steering_wheel_angle/min")
    """Is always 'summary/steering_wheel_angle/min'"""


@dataclass
class Summary__SteeringWheelAngle__Min__UStdDev(NumberData):
    """Uncertainty for `Summary__SteeringWheelAngle__Min` as standard deviation around the parent indicated value."""

    val: float = field(
        default_factory=lambda: no_default(field="Summary__SteeringWheelAngle__Min__UStdDev.val"), metadata=required
    )
    """Standard deviation around the parent indicated value. """

    name: Literal["summary/steering_wheel_angle/min/ustddev"] = field(default="summary/steering_wheel_angle/min/ustddev")
    """Is always 'summary/steering_wheel_angle/min/ustddev'."""


@dataclass
class Summary__SteeringAngle__Max(NumberData):
    """Maximum steering angle (i.e. tire angle) of the object over all frames in this file."""

    val: float = field(default_factory=lambda: no_default(field="Summary__SteeringAngle__Max.val"), metadata=required)
    """Maximum steering angle in radians, counter clockwise."""

    name: Literal["summary/steering_angle/max"] = field(default="summary/steering_angle/max")
    """Is always 'summary/steering_angle/max'"""


@dataclass
class Summary__SteeringAngle__Max__UStdDev(NumberData):
    """Uncertainty for `Summary__SteeringAngle__Max` as standard deviation around the parent indicated value."""

    val: float = field(default_factory=lambda: no_default(field="Summary__SteeringAngle__Max__UStdDev.val"), metadata=required)
    """Standard deviation around the parent indicated value. """

    name: Literal["summary/steering_angle/max/ustddev"] = field(default="summary/steering_angle/max/ustddev")
    """Is always 'summary/steering_angle/max/ustddev'."""


@dataclass
class Summary__SteeringAngle__Min(NumberData):
    """Minimum steering angle (i.e. tire angle) of the object over all frames in this file."""

    val: float = field(default_factory=lambda: no_default(field="Summary__SteeringAngle__Min"), metadata=required)
    """Minimum steering angle in radians, counter clockwise."""

    name: Literal["summary/steering_angle/min"] = field(default="summary/steering_angle/min")
    """Is always 'summary/steering_angle/min'"""


@dataclass
class Summary__SteeringAngle__Min__UStdDev(NumberData):
    """Uncertainty for `Summary__SteeringAngle__Min` as standard deviation around the parent indicated value."""

    val: float = field(default_factory=lambda: no_default(field="Summary__SteeringAngle__Min__UStdDev.val"), metadata=required)
    """Standard deviation around the parent indicated value. """

    name: Literal["summary/steering_angle/min/ustddev"] = field(default="summary/steering_angle/min/ustddev")
    """Is always 'summary/steering_angle/min/ustddev'."""


@dataclass
class Summary__Coordinates__ScenarioStart(VectorData):
    """Coordinate of the first frame in this file (in world coordinates)"""

    val: tuple[float, float, float] = field(
        default_factory=lambda: no_default(field="Summary__Coordinates__ScenarioStart.val"), metadata=required
    )
    """The (x, y, z) coordinates of the first frame of this object (m)."""

    name: Literal["summary/coordinates/scenario_start"] = field(default="summary/coordinates/scenario_start")
    """Is always 'summary/coordinates/scenario_start'"""


@dataclass
class Summary__Coordinates__ScenarioStart__UStdDev(VectorData):
    """Uncertainty for `Summary__Coordinates__ScenarioStart` as standard deviation around the parent indicated value."""

    val: tuple[float, float, float] = field(
        default_factory=lambda: no_default(field="Summary__Coordinates__ScenarioStart__UStdDev.val"), metadata=required
    )
    """Standard deviation around the parent indicated value. """

    name: Literal["summary/coordinates/scenario_start/ustddev"] = field(default="summary/coordinates/scenario_start/ustddev")
    """Is always 'summary/coordinates/scenario_start/ustddev'."""


@dataclass
class Summary__Coordinates__ScenarioEnd(VectorData):
    """Coordinate of the last frame in this file (in world coordinates)"""

    val: tuple[float, float, float] = field(
        default_factory=lambda: no_default(field="Summary__Coordinates__ScenarioEnd.val"), metadata=required
    )
    """The (x, y, z) coordinates of the last frame of this object (m)."""

    name: Literal["summary/coordinates/scenario_end"] = field(default="summary/coordinates/scenario_end")
    """Is always 'summary/coordinates/scenario_end'"""


@dataclass
class Summary__Coordinates__ScenarioEnd__UStdDev(VectorData):
    """Uncertainty for `Summary__Coordinates__ScenarioEnd` as standard deviation around the parent indicated value."""

    val: tuple[float, float, float] = field(
        default_factory=lambda: no_default(field="Summary__Coordinates__ScenarioEnd__UStdDev.val"), metadata=required
    )
    """Standard deviation around the parent indicated value. """

    name: Literal["summary/coordinates/scenario_end/ustddev"] = field(default="summary/coordinates/scenario_end/ustddev")
    """Is always 'summary/coordinates/scenario_end/ustddev'."""



"""Dynamic attributes of vehicles that are rideable"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass, field
from typing import Union

from apischema.metadata import required
from uai_openlabel import ObjectData as BaseObjectData
from uai_openlabel import no_default

from aveas_openlabel.attribute_enforcer import EachAttributeOnlyOnceEnforcer
from aveas_openlabel.attributes.general import (
    Acceleration,
    Acceleration__UStdDev,
    BestDetectedPoint,
    BestDetectedSide,
    BoundingBox,
    BoundingBox__UStdDev,
    Velocity,
    Velocity__UStdDev,
)
from aveas_openlabel.attributes.hmi_feedback import (
    HmiFeedback__Acoustic,
    HmiFeedback__Other,
    HmiFeedback__Visual,
)
from aveas_openlabel.attributes.impact import (
    Impact__gTTC__ObjectIds,
    Impact__gTTC__Values,
    Impact__PrET__ObjectIds,
    Impact__PrET__Values,
    Impact__THW__ObjectIds,
    Impact__THW__Values,
)
from aveas_openlabel.attributes.interior import (
    Interior__AcceleratorPedal,
    Interior__AutomatedControl__Lateral,
    Interior__AutomatedControl__Longitudinal,
    Interior__BrakePedal,
    Interior__Gear,
    Interior__HasRider,
    Interior__SteeringAngle,
    Interior__SteeringAngle__UStdDev,
    Interior__Wiper,
)
from aveas_openlabel.attributes.lights import (
    Lights__Brake,
    Lights__Daytime,
    Lights__Front,
    Lights__HighBeam,
    Lights__Indicator__Left,
    Lights__Indicator__Right,
)
from aveas_openlabel.attributes.open_drive import (
    OpenDrive__LaneId,
    OpenDrive__LanePosition,
    OpenDrive__LocalRoadCoordinates,
    OpenDrive__LocalRoadCoordinates__UStdDev,
    OpenDrive__RoadId,
)
from aveas_openlabel.attributes.operator import (
    Operator__FocussedObject,
    Operator__FocussedObject__Id,
    Operator__FocussedObject__Uncertainties,
    Operator__FocussedPoint,
    Operator__FocussedPoint__UStdDev,
    Operator__HandInteractionArea,
    Operator__HeadRotation,
    Operator__HeadRotation__UStdDev,
    Operator__Pupil,
    Operator__Pupil__UStdDev,
    Operator__SixDoFRotationAndAcceleration,
    Operator__ViewingAngle,
)
from aveas_openlabel.attributes.road import (
    Road__Classification,
    Road__NumberLanes__Left__Legal,
    Road__NumberLanes__Left__Physical,
    Road__NumberLanes__Right__Legal,
    Road__NumberLanes__Right__Physical,
    Road__SpeedLimit,
)
from aveas_openlabel.attributes.traffic import (
    Traffic__Density,
    Traffic__Density__UStdDev,
    Traffic__Volume,
    Traffic__Volume__UStdDev,
)


@dataclass
class ObjectInFrameData__All(BaseObjectData, EachAttributeOnlyOnceEnforcer):
    """
    This class contains the dynamic attributes of rideable vehicles.

    They can be retrieved from the `cuboid`, `boolean`, `num`, `text`, and `vec` fields according to their type.
    """

    boolean: list[
        Union[
            Interior__AutomatedControl__Lateral,
            Interior__AutomatedControl__Longitudinal,
            Interior__HasRider,
            Interior__Wiper,
            Lights__Brake,
            Lights__Indicator__Left,
            Lights__Indicator__Right,
            Lights__Front,
            Lights__Daytime,
            Lights__HighBeam,
        ]
    ] = field(default_factory=lambda: no_default(field="ObjectInFrameData__All.boolean"), metadata=required)
    """Contains all boolean attributes"""

    cuboid: list[BoundingBox] = field(
        default_factory=lambda: no_default(field="ObjectInFrameData__All.cuboid"), metadata=required
    )
    """Contains a single cuboid geometry"""

    num: list[
        Union[
            HmiFeedback__Visual,
            HmiFeedback__Acoustic,
            HmiFeedback__Other,
            Interior__AcceleratorPedal,
            Interior__BrakePedal,
            Interior__Gear,
            Interior__SteeringAngle,
            Interior__SteeringAngle__UStdDev,
            Operator__HeadRotation,
            Operator__HeadRotation__UStdDev,
            Operator__ViewingAngle,
            Traffic__Density,
            Traffic__Density__UStdDev,
            Traffic__Volume,
            Traffic__Volume__UStdDev,
            Road__NumberLanes__Left__Legal,
            Road__NumberLanes__Left__Physical,
            Road__NumberLanes__Right__Legal,
            Road__NumberLanes__Right__Physical,
            Road__SpeedLimit,
            OpenDrive__LanePosition,
        ]
    ] = field(default_factory=lambda: no_default(field="ObjectInFrameData__All.num"), metadata=required)
    """Contains all numeric attributes"""

    text: list[
        Union[
            BestDetectedSide,
            Operator__FocussedObject,
            Operator__FocussedObject__Id,
            OpenDrive__RoadId,
            OpenDrive__LaneId,
            Road__Classification,
        ]
    ] = field(default_factory=lambda: no_default(field="ObjectInFrameData__All.text"), metadata=required)
    """Contains all textual attributes"""

    vec: list[
        Union[
            BestDetectedPoint,
            BoundingBox__UStdDev,
            Velocity,
            Velocity__UStdDev,
            Acceleration,
            Acceleration__UStdDev,
            OpenDrive__LocalRoadCoordinates,
            OpenDrive__LocalRoadCoordinates__UStdDev,
            Operator__FocussedObject__Uncertainties,
            Operator__FocussedPoint,
            Operator__FocussedPoint__UStdDev,
            Operator__Pupil,
            Operator__Pupil__UStdDev,
            Operator__HandInteractionArea,
            Operator__SixDoFRotationAndAcceleration,
            Impact__gTTC__Values,
            Impact__gTTC__ObjectIds,
            Impact__PrET__Values,
            Impact__PrET__ObjectIds,
            Impact__THW__ObjectIds,
            Impact__THW__Values,
        ]
    ] = field(default_factory=lambda: no_default(field="ObjectInFrameData__All.vec"), metadata=required)
    """Contains all vectorial attributes"""



"""The `ScenarioContext` class and its attributes

The `ScenarioContext` class contains `ScenarioContextData` which provides information about the circumstances of the scenario.
"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass, field
from typing import Literal, Union

from apischema.metadata import required
from uai_openlabel import (
    Attributes,
    no_default,
)
from uai_openlabel import (
    Context as BaseContext,
)

from aveas_openlabel.attribute_enforcer import EachAttributeOnlyOnceEnforcer
from aveas_openlabel.contexts.scenario_context_data import (
    Scenario__ContainsHighway,
    Scenario__ContainsRural,
    Scenario__ContainsUrban,
    Scenario__Course,
    Scenario__End__Coordinates,
    Scenario__End__Location,
    Scenario__IsBiased,
    Scenario__IsSampled,
    Scenario__IsSampled__ReferenceToSourceScenario,
    Scenario__IsStaged,
    Scenario__MaximumVehicleSpeed,
    Scenario__MaximumVehicleSpeed__Frame,
    Scenario__MaximumVehicleSpeed__UStdDev,
    Scenario__MinimumVehicleDistanceS,
    Scenario__MinimumVehicleDistanceS__Frame,
    Scenario__MinimumVehicleDistanceS__UStdDev,
    Scenario__MinimumVehicleSpeed,
    Scenario__MinimumVehicleSpeed__Frame,
    Scenario__MinimumVehicleSpeed__UStdDev,
    Scenario__RatioAverageSpeedToSpeedLimit,
    Scenario__RatioAverageSpeedToSpeedLimit__UStdDev,
    Scenario__Start__Coordinates,
    Scenario__Start__Location,
    Scenario__WeekdayNumber,
)


@dataclass
class ScenarioContextData(Attributes, EachAttributeOnlyOnceEnforcer):
    """
    This class contains the attributes of the `ScenarioContext` class.

    They can be retrieved from the `boolean`, `num`, `text`, and `vec` fields according to their type.
    """

    boolean: list[
        Union[
            Scenario__ContainsRural,
            Scenario__ContainsUrban,
            Scenario__ContainsHighway,
            Scenario__IsSampled,
            Scenario__IsBiased,
            Scenario__IsStaged,
        ]
    ] = field(default_factory=lambda: no_default(field="ScenarioContextData.boolean"), metadata=required)
    """The boolean attributes of the `ScenarioContext`"""

    num: list[
        Union[
            Scenario__MinimumVehicleSpeed,
            Scenario__MinimumVehicleSpeed__UStdDev,
            Scenario__MaximumVehicleSpeed,
            Scenario__MaximumVehicleSpeed__UStdDev,
            Scenario__MinimumVehicleDistanceS,
            Scenario__MinimumVehicleDistanceS__UStdDev,
            Scenario__WeekdayNumber,
            Scenario__RatioAverageSpeedToSpeedLimit,
            Scenario__RatioAverageSpeedToSpeedLimit__UStdDev,
        ]
    ] = field(default_factory=lambda: no_default(field="ScenarioContextData.num"), metadata=required)
    """The numeric attributes of the `ScenarioContext`"""

    text: list[
        Union[
            Scenario__Start__Location,
            Scenario__End__Location,
            Scenario__IsSampled__ReferenceToSourceScenario,
            Scenario__MinimumVehicleSpeed__Frame,
            Scenario__MaximumVehicleSpeed__Frame,
            Scenario__MinimumVehicleDistanceS__Frame,
        ]
    ] = field(default_factory=lambda: no_default(field="ScenarioContextData.text"), metadata=required)
    """The textual attributes of the `ScenarioContext`"""

    vec: list[Union[Scenario__Start__Coordinates, Scenario__End__Coordinates, Scenario__Course]] = field(
        default_factory=lambda: no_default(field="ScenarioContextData.vec"), metadata=required
    )
    """The vectorial attributes of the `ScenarioContext`"""


@dataclass
class ScenarioContext(BaseContext):
    """
    An instance of this class is found in the `contexts` dict of the root `AveasOpenLabel` instance.
    """

    name: Literal["scenario_context"] = field(default="scenario_context")
    """Is always 'scenario_context'."""

    type: Literal["ScenarioContext"] = field(default="ScenarioContext")
    """Is always 'ScenarioContext', the Python class representing this object."""

    context_data: ScenarioContextData = field(
        default_factory=lambda: no_default(field="ScenarioContext.context_data"), metadata=required
    )
    """Contains the attributes of the `ScenarioContext`."""



"""Information on occurring collisions with other objects"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass, field
from typing import Literal

from apischema.metadata import required
from uai_openlabel import ObjectUid, TextData, VectorData, no_default


@dataclass
class Impact__Point(VectorData):  # TODO: Koordinatensystem wie spezifizieren?
    """Point of first contact of the first collision (in this object's coordinates)"""

    val: tuple[float, float] = field(default_factory=lambda: no_default(field="Impact__Point.val"), metadata=required)
    """The (x, y) coordinates of the collision point in (m)."""

    name: Literal["impact/point"] = field(default="impact/point")
    """Is always 'impact/point'"""


@dataclass
class Impact__Point__UStdDev(VectorData):
    """Uncertainty for `Impact__Point` as standard deviation around the parent indicated value."""

    val: tuple[float, float] = field(default_factory=lambda: no_default(field="Impact__Point__UStdDev.val"), metadata=required)
    """Standard deviation around the parent indicated value. """

    name: Literal["impact/point/ustddev"] = field(default="impact/point/ustddev")
    """Is always 'impact/point/ustddev'."""


@dataclass
class Impact__Velocity(VectorData):  # TODO: Koordinatensystem wie spezifizieren?
    """Relative velocity of the first contact of the first collision (in this object's coordinates)"""

    val: tuple[float, float] = field(default_factory=lambda: no_default(field="Impact__Velocity.val"), metadata=required)
    """The (x, y) components of impact velocity in (m/s)."""

    name: Literal["impact/velocity"] = field(default="impact/velocity")
    """Is always 'impact/velocity'"""


@dataclass
class Impact__Velocity__UStdDev(VectorData):
    """Uncertainty for `Impact__Velocity` as standard deviation around the parent indicated value."""

    val: tuple[float, float] = field(
        default_factory=lambda: no_default(field="Impact__Velocity__UStdDev.val"), metadata=required
    )
    """Standard deviations around the parent indicated values. """

    name: Literal["impact/velocity/ustddev"] = field(default="impact/velocity/ustddev")
    """Is always 'impact/velocity/ustddev'."""


@dataclass
class Impact__Frame(TextData):
    """Frame ID of the first contact of the first collision."""

    val: str = field(default_factory=lambda: no_default(field="Impact__Frame.val"), metadata=required)
    """The frame ID."""

    name: Literal["impact/frame"] = field(default="impact/frame")
    """Is always 'impact/frame'"""


@dataclass
class Impact__gTTC__ObjectIds(VectorData):
    """
    Object IDs of the opposing traffic participants with a geometrical time-to-collision (gTTC) lower than
    the value "threshold_gttc" in `Metadata`.
    """

    val: tuple[ObjectUid, ...] = field(
        default_factory=lambda: no_default(field="Impact__gTTC__ObjectIds.val"), metadata=required
    )
    """The object IDs of the opposing traffic participants."""

    name: Literal["impact/gttc/object_ids"] = field(default="impact/gttc/object_ids")
    """Is always 'impact/gttc/object_ids'."""


@dataclass
class Impact__gTTC__Values(VectorData):
    """Values of the geometrical time-to-collision (gTTC) for the traffic participants mentioned in `Impact__gTTC__ObjectIds`."""

    val: tuple[float, ...] = field(default_factory=lambda: no_default(field="Impact__gTTC__Values.val"), metadata=required)
    """Values of the geometrical time-to-collision (gTTC)."""

    name: Literal["impact/gttc/values"] = field(default="impact/gttc/values")
    """Is always 'impact/gttc/values'."""


@dataclass
class Impact__PrET__ObjectIds(VectorData):
    """
    Object IDs of the opposing traffic participants with a predicted encroachment time (PrET) lower than
    the value "threshold_pret" in `Metadata`.
    """

    val: tuple[ObjectUid, ...] = field(
        default_factory=lambda: no_default(field="Impact__PrET__ObjectIds.val"), metadata=required
    )
    """The object IDs of the opposing traffic participants."""

    name: Literal["impact/pret/object_ids"] = field(default="impact/pret/object_ids")
    """Is always 'impact/pret/object_ids'."""


@dataclass
class Impact__PrET__Values(VectorData):
    """Values of the predicted encroachment time (PrET) for the traffic participants mentioned in `Impact__PrET__ObjectIds`."""

    val: tuple[float, ...] = field(default_factory=lambda: no_default(field="Impact__PrET__Values.val"), metadata=required)
    """Values of the predicted encroachment time (PrET)."""

    name: Literal["impact/pret/values"] = field(default="impact/pret/values")
    """Is always 'impact/pret/values'."""


@dataclass
class Impact__THW__ObjectIds(VectorData):
    """
    Object IDs of the opposing traffic participants with a time headway (THW) lower than
    the value "threshold_thw" in `Metadata`.
    """

    val: tuple[ObjectUid, ...] = field(
        default_factory=lambda: no_default(field="Impact__THW__ObjectIds.val"), metadata=required
    )
    """The object IDs of the opposing traffic participants."""

    name: Literal["impact/thw/object_ids"] = field(default="impact/thw/object_ids")
    """Is always 'impact/thw/object_ids'."""


@dataclass
class Impact__THW__Values(VectorData):
    """Values of the time headway (THW) for the traffic participants mentioned in `Impact__THW__ObjectIds`."""

    val: tuple[float, ...] = field(default_factory=lambda: no_default(field="Impact__THW__Values.val"), metadata=required)
    """Values of the time headway (THW)."""

    name: Literal["impact/thw/values"] = field(default="impact/thw/values")
    """Is always 'impact/thw/values'."""



"""The `EnvironmentContext` class and its attributes

The `EnvironmentContext` class contains `EnvironmentContextData` which provides information about the circumstances of the environment.
"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass, field
from typing import Literal, Union

from apischema.metadata import required
from uai_openlabel import (
    Attributes,
    no_default,
)
from uai_openlabel import (
    Context as BaseContext,
)

from aveas_openlabel.attribute_enforcer import EachAttributeOnlyOnceEnforcer
from aveas_openlabel.contexts.environment_context_data import (
    Environment__CloudCover,
    Environment__CloudCover__URadius,
    Environment__LightingConditions,
    Environment__LightingConditions__Uncertainties,
    Environment__PrecipitationIntensity,
    Environment__PrecipitationIntensity__UStdDev,
    Environment__RoadCondition,
    Environment__RoadCondition__Uncertainties,
    Environment__Temperature,
    Environment__Temperature__UStdDev,
    Environment__VisibilityRange,
    Environment__VisibilityRange__UStdDev,
    Environment__Wind__BeaufortForce,
    Environment__Wind__BeaufortForce__URadius,
    Environment__Wind__Heading,
    Environment__Wind__Heading__UStdDev,
)


@dataclass
class EnvironmentContextData(Attributes, EachAttributeOnlyOnceEnforcer):
    """
    This class contains the attributes of the `EnvironmentContext` class.

    They can be retrieved from the `num` and `text` fields according to their type.
    """

    num: list[
        Union[
            Environment__CloudCover,
            Environment__CloudCover__URadius,
            Environment__PrecipitationIntensity,
            Environment__PrecipitationIntensity__UStdDev,
            Environment__Temperature,
            Environment__Temperature__UStdDev,
            Environment__VisibilityRange,
            Environment__VisibilityRange__UStdDev,
            Environment__Wind__BeaufortForce,
            Environment__Wind__BeaufortForce__URadius,
            Environment__Wind__Heading,
            Environment__Wind__Heading__UStdDev,
        ]
    ] = field(default_factory=lambda: no_default(field="EnvironmentContextData.num"), metadata=required)
    """The numeric attributes of the `EnvironmentContext`"""

    text: list[
        Union[
            Environment__LightingConditions,
            Environment__RoadCondition,
        ]
    ] = field(default_factory=lambda: no_default(field="EnvironmentContextData.text"), metadata=required)
    """The textual attributes of the `EnvironmentContext`"""

    vec: list[
        Union[
            Environment__LightingConditions__Uncertainties,
            Environment__RoadCondition__Uncertainties,
        ]
    ] = field(default_factory=lambda: no_default(field="EnvironmentContextData.vec"), metadata=required)
    """The vectorial attributes of the `EnvironmentContext`"""


@dataclass
class EnvironmentContext(BaseContext):
    """
    An instance of this class is found in the `contexts` dict of the root `AveasOpenLabel` instance.
    """

    name: Literal["environment_context"] = field(default="environment_context")
    """Is always 'environment_context'."""

    type: Literal["EnvironmentContext"] = field(default="EnvironmentContext")
    """Is always 'EnvironmentContext', the Python class representing this object."""

    context_data: EnvironmentContextData = field(
        default_factory=lambda: no_default(field="EnvironmentContext.context_data"), metadata=required
    )
    """Contains the attributes of the `EnvironmentContext`."""



"""Implementation of attributes in `EnvironmentContextData`

"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass, field
from enum import Enum
from typing import Literal

from apischema.metadata import required
from uai_openlabel import (
    NumberData,
    TextData,
    VectorData,
    no_default,
)


class Environment__LightingConditionsValue(str, Enum):
    """Possible values of 'lighting_conditions'"""

    DARKNESS = "darkness"
    """No significant primary lighting. Includes night time, as well as e.g. un-illuminated parking garages"""

    ARTIFICIAL = "artificial"
    """Artificial primary lighting"""

    CIVIL_TWILIGHT = "civil_twilight"
    """
    Defined as "the period of incomplete darkness when the upper limb of the sun is below the visible horizon, 
    and the center of the sun is not more than 6° below the celestial horizon". (Glossary of Marine Navigation / 
    Definitions from the US Astronomical Applications Dept.)
    """

    DAY = "day"
    """Any natural primary lighting that is not darkness or civil_twilight"""


@dataclass
class Environment__LightingConditions(TextData):
    """Primary (i.e. dominant) lighting condition that continuously and statically illuminates the relevant traffic
    area."""

    val: Environment__LightingConditionsValue = field(
        default_factory=lambda: no_default(field="Environment__LightingConditions.val"), metadata=required
    )
    """See `Environment__LightingConditionsValue` for possible values. """

    name: Literal["lighting_conditions"] = field(default="lighting_conditions")
    """Is always 'lighting_conditions'"""


@dataclass
class Environment__LightingConditions__Uncertainties(VectorData):
    """
    Uncertainty values for `Environment__LightingConditions` (must accumulate to 1)
    """

    val: tuple[float, float, float, float] = field(
        default_factory=lambda: no_default(field="Environment__LightingConditions__Uncertainties.val"), metadata=required
    )
    """Tuple with class probabilities (must accumulate to 1), in the following order: 
     - darkness
     - artificial
     - civil_twilight
     - day
    """

    name: Literal["environment/lighting_conditions/uncertainties"] = field(
        default="environment/lighting_conditions/uncertainties"
    )
    """Is always 'environment/lighting_conditions/uncertainties'"""


class Environment__RoadConditionValue(str, Enum):
    """Possible values of `Environment__RoadCondition.val`"""

    DRY = "dry"
    """Dry road condition (with good friction)"""

    WET = "wet"
    """Wet road condition (liquid water on the surface causing reduced friction)"""

    SLIPPERY = "slippery"
    """Slippery road conditions not caused by wetness, e.g. ice or leaves"""


@dataclass
class Environment__RoadCondition(TextData):
    """Describes the meteorological road surface condition."""

    val: Environment__RoadConditionValue = field(
        default_factory=lambda: no_default(field="Environment__RoadCondition.val"), metadata=required
    )
    """See `Environment__RoadConditionValue` for possible values. """

    name: Literal["road_condition"] = field(default="road_condition")
    """Is always 'road_condition'"""


@dataclass
class Environment__RoadCondition__Uncertainties(VectorData):
    """
    Uncertainty values for `Environment__RoadCondition` (must accumulate to 1)
    """

    val: tuple[float, float, float] = field(
        default_factory=lambda: no_default(field="Environment__RoadCondition__Uncertainties.val"), metadata=required
    )
    """Tuple with class probabilities (must accumulate to 1), in the following order: 
     - dry
     - wet
     - slippery
    """

    name: Literal["environment/road_condition/uncertainties"] = field(default="environment/road_condition/uncertainties")
    """Is always 'environment/road_condition/uncertainties'"""


@dataclass
class Environment__VisibilityRange(NumberData):
    """Visibility range in (m). Values can be interpreted based on Meteorological Office 1969 according to Allen Howard Perry and
    Leslie Symons. Highway Meteorology. Taylor & Francis, 2003:

    - > 40 km: excellent
    - 10-40 km: good
    - 4–10 km: moderate
    - 2–4 km: poor
    - 1–2 km: mist
    - 200–1000 m: fog
    - 40–200 m: thick fog
    - < 40 m: dense fog
    """

    val: int = field(default_factory=lambda: no_default(field="Environment__VisibilityRange.val"), metadata=required)
    """Visibility range in (m)."""

    name: Literal["environment/visibility_range"] = field(default="environment/visibility_range")
    """Is always 'environment/visibility_range'"""


@dataclass
class Environment__VisibilityRange__UStdDev(NumberData):
    """Uncertainty for `Environment__VisibilityRange` as standard deviation around the parent indicated value."""

    val: float = field(default_factory=lambda: no_default(field="Environment__VisibilityRange__UStdDev.val"), metadata=required)
    """Standard deviation around the parent indicated value. """

    name: Literal["environment/visibility_range/ustddev"] = field(default="environment/visibility_range/ustddev")
    """Is always 'environment/visibility_range/ustddev'. """


@dataclass
class Environment__PrecipitationIntensity(NumberData):
    """Precipitation intensity in (mm/h). Values can be interpreted based on the Glossary of Meteorology (2012). "Rain". American
    Meteorological Society:

    - 0 mm/h: none
    - 0-2.5 mm/h: light rain
    - 2.6-7.6 mm/h: moderate rain
    - 7.6-50 mm/h: heavy rain
    - 50-100 mm/h: violent rain
    - > 100 mm/h: cloudburst
    """

    val: int = field(default_factory=lambda: no_default(field="Environment__PrecipitationIntensity.val"), metadata=required)
    """Precipitation intensity in (mm/h). """

    name: Literal["environment/precipitation_intensity"] = field(default="environment/precipitation_intensity")
    """Is always 'environment/precipitation_intensity'. """


@dataclass
class Environment__PrecipitationIntensity__UStdDev(NumberData):
    """Uncertainty for `Environment__PrecipitationIntensity` as standard deviation around the parent indicated value."""

    val: float = field(
        default_factory=lambda: no_default(field="Environment__PrecipitationIntensity__UStdDev.val"), metadata=required
    )
    """Standard deviation around the parent indicated value. """

    name: Literal["environment/precipitation_intensity/ustddev"] = field(default="environment/precipitation_intensity/ustddev")
    """Is always 'environment/precipitation_intensity/ustddev'. """


@dataclass
class Environment__Temperature(NumberData):
    """Air temperature in degrees Celsius."""

    val: int = field(default_factory=lambda: no_default(field="Environment__Temperature.val"), metadata=required)
    """The air temperature is rounded to the nearest integer. """

    name: Literal["environment/temperature_celsius"] = field(default="environment/temperature_celsius")
    """Is always 'environment/temperature_celsius'. """


@dataclass
class Environment__Temperature__UStdDev(NumberData):
    """Uncertainty for `Environment__Temperature` as standard deviation around the parent indicated value."""

    val: float = field(default_factory=lambda: no_default(field="Environment__Temperature__UStdDev.val"), metadata=required)
    """Standard deviation around the parent indicated value. """

    name: Literal["environment/temperature/ustddev"] = field(default="environment/temperature/ustddev")
    """Is always 'environment/temperature/ustddev'. """


@dataclass
class Environment__Wind__BeaufortForce(NumberData):
    """
    Wind speed by Beaufort wind force scale:

    - 0: Calm (2 km/h)
    - 1: Light air (2–5 km/h)
    - 2: Light breeze (6–11 km/h)
    - 3: Gentle breeze (12–19 km/h)
    - 4: Moderate breeze (20–28 km/h)
    - 5: Fresh breeze (29–38 km/h)
    - 6: Strong breeze (39–49 km/h)
    - 7: High wind (50–61 km/h)
    - 8: Gale (62–74 km/h)
    - 9: Strong gale (75–88 km/h)
    - 10: Storm (89–102 km/h)
    - 11: Violent storm (103–117 km/h)
    - 12: Hurricane-force (≥ 118 km/h)
    """

    val: int = field(default_factory=lambda: no_default(field="Environment__Wind__BeaufortForce.val"), metadata=required)
    """Wind speed by Beaufort wind force scale"""

    name: Literal["environment/wind/beaufort_force"] = field(default="environment/wind/beaufort_force")
    """Is always 'environment/wind/beaufort_force'"""


@dataclass
class Environment__Wind__BeaufortForce__URadius(NumberData):
    """Uncertainty for `Environment__Wind__BeaufortForce` as radius (plus/minus this value) around the parent indicated value."""

    val: float = field(
        default_factory=lambda: no_default(field="Environment__Wind__BeaufortForce__URadius.val"), metadata=required
    )
    """Uncertainty radius around the parent indicated value. """

    name: Literal["environment/wind/beaufort_force/uradius"] = field(default="environment/wind/beaufort_force/uradius")
    """Is always 'environment/wind/beaufort_force/uradius'."""


@dataclass
class Environment__Wind__Heading(NumberData):
    """Wind heading"""

    val: float = field(default_factory=lambda: no_default(field="Environment__Wind__Heading.val"), metadata=required)
    """Wind heading in radians, CCW with 0 = east according to ENU coordinates."""

    name: Literal["environment/wind/heading"] = field(default="environment/wind/heading")
    """Is always 'environment/wind/heading'"""


@dataclass
class Environment__Wind__Heading__UStdDev(NumberData):
    """Uncertainty for `Environment__Wind__Heading` as standard deviation around the parent indicated value."""

    val: float = field(default_factory=lambda: no_default(field="Environment__Wind__Heading__UStdDev.val"), metadata=required)
    """Standard deviation around the parent indicated value. """

    name: Literal["environment/wind/heading/ustddev"] = field(default="environment/wind/heading/ustddev")
    """Is always 'environment/wind/heading/ustddev'."""


@dataclass
class Environment__CloudCover(NumberData):
    """Cloud cover of the sky."""

    val: int = field(default_factory=lambda: no_default(field="Environment__CloudCover.val"), metadata=required)
    """
    Cloud cover in oktas, i.e. between 0 (completely clear) and 8 (completely cloudy). A value of 9 denotes that the
    sky is obstructed from the view, e.g. in a parking garage.
    """

    name: Literal["environment/cloud_cover"] = field(default="environment/cloud_cover")
    """Is always 'environment/cloud_cover'"""


@dataclass
class Environment__CloudCover__URadius(NumberData):
    """Uncertainty for `Environment__CloudCover` as radius (plus/minus this value) around the parent indicated value."""

    val: float = field(default_factory=lambda: no_default(field="Environment__CloudCover__URadius.val"), metadata=required)
    """Uncertainty radius around the parent indicated value. """

    name: Literal["environment/cloud_cover/uradius"] = field(default="environment/cloud_cover/uradius")
    """Is always 'environment/cloud_cover/uradius'."""



"""Static attributes of vehicles"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass, field
from typing import Union

from apischema.metadata import required
from uai_openlabel import ObjectData as BaseObjectData
from uai_openlabel import no_default

from aveas_openlabel.attribute_enforcer import EachAttributeOnlyOnceEnforcer
from aveas_openlabel.attributes.dimension import (
    Dimensions__CenterOfGravity,
    Dimensions__CenterOfGravity__UStdDev,
    Dimensions__Size,
    Dimensions__Size__UStdDev,
)
from aveas_openlabel.attributes.general import Classification__Uncertainties, IsRecorder
from aveas_openlabel.attributes.impact import (
    Impact__Frame,
    Impact__Point,
    Impact__Point__UStdDev,
    Impact__Velocity,
    Impact__Velocity__UStdDev,
)
from aveas_openlabel.attributes.operator import (
    Operator__Age,
    Operator__BodyHeight,
    Operator__Gender,
    Operator__Personality,
)
from aveas_openlabel.attributes.summary import (
    Summary__Accel__Max,
    Summary__Accel__Max__UStdDev,
    Summary__Accel__Min,
    Summary__Accel__Min__UStdDev,
    Summary__Coordinates__ScenarioEnd,
    Summary__Coordinates__ScenarioEnd__UStdDev,
    Summary__Coordinates__ScenarioStart,
    Summary__Coordinates__ScenarioStart__UStdDev,
    Summary__Speed__Max,
    Summary__Speed__Max__UStdDev,
    Summary__Speed__Min,
    Summary__Speed__Min__UStdDev,
    Summary__SteeringAngle__Max,
    Summary__SteeringAngle__Max__UStdDev,
    Summary__SteeringAngle__Min,
    Summary__SteeringAngle__Min__UStdDev,
    Summary__SteeringWheelAngle__Max,
    Summary__SteeringWheelAngle__Max__UStdDev,
    Summary__SteeringWheelAngle__Min,
    Summary__SteeringWheelAngle__Min__UStdDev,
)


@dataclass
class ObjectData__Unattached(BaseObjectData, EachAttributeOnlyOnceEnforcer):
    """
    This class contains the static attributes of vehicle classes.

    They can be retrieved from the `boolean`, `num`, `text` and `vec` fields according to their type.
    """

    boolean: list[Union[IsRecorder]] = field(
        default_factory=lambda: no_default(field="ObjectData__Unattached.boolean"), metadata=required
    )
    """Contains all boolean attributes"""

    num: list[
        Union[
            Operator__Age,
            Operator__BodyHeight,
            Summary__Speed__Max,
            Summary__Speed__Max__UStdDev,
            Summary__Speed__Min,
            Summary__Speed__Min__UStdDev,
            Summary__Accel__Max,
            Summary__Accel__Max__UStdDev,
            Summary__Accel__Min,
            Summary__Accel__Min__UStdDev,
            Summary__SteeringWheelAngle__Max,
            Summary__SteeringWheelAngle__Max__UStdDev,
            Summary__SteeringWheelAngle__Min,
            Summary__SteeringWheelAngle__Min__UStdDev,
            Summary__SteeringAngle__Max,
            Summary__SteeringAngle__Max__UStdDev,
            Summary__SteeringAngle__Min,
            Summary__SteeringAngle__Min__UStdDev,
        ]
    ] = field(default_factory=lambda: no_default(field="ObjectData__Unattached.num"), metadata=required)
    """Contains all numeric attributes"""

    text: list[
        Union[
            Impact__Frame,
            Operator__Gender,
        ]
    ] = field(default_factory=lambda: no_default(field="ObjectData__Unattached.text"), metadata=required)
    """Contains all textual attributes"""

    vec: list[
        Union[
            Classification__Uncertainties,
            Dimensions__Size,
            Dimensions__Size__UStdDev,
            Dimensions__CenterOfGravity,
            Dimensions__CenterOfGravity__UStdDev,
            Impact__Point,
            Impact__Point__UStdDev,
            Impact__Velocity,
            Impact__Velocity__UStdDev,
            Operator__Personality,
            Summary__Coordinates__ScenarioStart,
            Summary__Coordinates__ScenarioStart__UStdDev,
            Summary__Coordinates__ScenarioEnd,
            Summary__Coordinates__ScenarioEnd__UStdDev,
        ]
    ] = field(default_factory=lambda: no_default(field="ObjectData__Unattached.vec"), metadata=required)
    """Contains all vectorial attributes"""



"""Attributes referencing the object to the OpenDRIVE map"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass, field
from typing import Literal

from apischema.metadata import required
from uai_openlabel import (
    NumberData,
    TextData,
    VectorData,
    no_default,
)


@dataclass
class OpenDrive__RoadId(TextData):
    """Road ID in the OpenDRIVE file that the local road coordinates are referencing"""

    val: str = field(default_factory=lambda: no_default(field="OpenDrive__RoadId.val"), metadata=required)
    """The OpenDRIVE road ID"""

    name: Literal["used_road_link"] = field(default="used_road_link")
    """Is always 'open_drive/road_id'"""


@dataclass
class OpenDrive__LaneId(TextData):
    """Lane ID the object is moving on, corresponding to the Road ID (stated in `OpenDrive__RoadId`) in the OpenDRIVE file"""

    val: str = field(default_factory=lambda: no_default(field="OpenDrive__LaneId.val"), metadata=required)
    """The OpenDRIVE lane ID"""

    name: Literal["open_drive/lane_id"] = field(default="open_drive/lane_id")
    """Is always 'open_drive/lane_id'"""


@dataclass
class OpenDrive__LanePosition(NumberData):
    """Relative lane position of the object which is moving on the corresponding Lane ID (stated in `OpenDrive__LaneId`) in the OpenDRIVE file. Range between -0.5 (right border) and 0.5 (left border)"""

    val: float = field(default_factory=lambda: no_default(field="OpenDrive__LanePosition.val"), metadata=required)
    """Relative lane position of object on the OpenDRIVE lane"""

    name: Literal["open_drive/lane_position"] = field(default="open_drive/lane_position")
    """Is always 'open_drive/lane_position'"""

    def __post_init__(self) -> None:
        if self.val < -0.5 or self.val > 0.5:
            raise ValueError("val can only be between -0.5 and 0.5.")


@dataclass
class OpenDrive__LocalRoadCoordinates(VectorData):
    """The bounding box position in the local road coordinates as defined in OpenDRIVE"""

    val: tuple[float, float] = field(
        default_factory=lambda: no_default(field="OpenDrive__LocalRoadCoordinates.val"), metadata=required
    )
    """The (s, t) tuple of the local road coordinates ."""

    name: Literal["open_drive/local_road_coordinates"] = field(default="open_drive/local_road_coordinates")
    """Is always 'open_drive/local_road_coordinates'"""


@dataclass
class OpenDrive__LocalRoadCoordinates__UStdDev(VectorData):
    """Uncertainty for `OpenDrive__LocalRoadCoordinates` as standard deviation around the parent indicated value."""

    val: tuple[float, float] = field(
        default_factory=lambda: no_default(field="OpenDrive__LocalRoadCoordinates__UStdDev.val"), metadata=required
    )
    """Standard deviation around the parent indicated value. """

    name: Literal["open_drive/local_road_coordinates/ustddev"] = field(default="open_drive/local_road_coordinates/ustddev")
    """Is always 'open_drive/local_road_coordinates/ustddev'."""



"""Object data of objects in the `AveasOpenLabel.frames` field."""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.



"""Attributes not belonging to any particular group"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import math
from dataclasses import dataclass, field
from enum import Enum
from typing import Literal, Union

from apischema.metadata import required
from uai_openlabel import (
    BooleanData,
    ObjectUid,
    TextData,
    VectorData,
    no_default,
)
from uai_openlabel import (
    ThreeDBoundingBoxEuler as BaseThreeDBoundingBoxEuler,
)


@dataclass
class BoundingBox(BaseThreeDBoundingBoxEuler):
    """A cuboid in 3D Euclidean space with a pose defined via Euler angles, according to ISO 8855."""

    val: tuple[float, float, float, float, float, float, float, float, float] = field(
        default_factory=lambda: no_default(field="BoundingBox.val"), metadata=required
    )
    """
    A tuple of nine floats. 
     - x in (m): The x coordinate of the 3D position of the center of the cuboid.
     - y in (m): The y coordinate of the 3D position of the center of the cuboid.
     - z in (m): The z coordinate of the 3D position of the center of the cuboid.
     - rx in (rad): Euler roll angle (positive: right roll).
     - ry in (rad): Euler pitch angles (positive: pitch down).
     - rz in (rad): Euler yaw angles (positive: yaw left).
     - sx in (m): The x (length) dimension of the cuboid.
     - sy in (m): The y (width) dimension of the cuboid.
     - sz in (m): The z (height) dimension of the cuboid.
    """

    name: Literal["bounding_box"] = field(default="bounding_box")
    """Is always 'bounding_box'."""


@dataclass
class BoundingBox__UStdDev(VectorData):
    """Uncertainty for `BoundingBox` as standard deviation around the parent indicated value."""

    val: tuple[float, float, float, float, float, float, float, float, float] = field(
        default_factory=lambda: no_default(field="BoundingBox__UStdDev.val"), metadata=required
    )
    """Standard deviation around the parent indicated value. """

    name: Literal["bounding_box/ustddev"] = field(default="bounding_box/ustddev")
    """Is always 'bounding_box/ustddev'."""


@dataclass
class Velocity(VectorData):
    """Velocity data of the object."""

    val: tuple[float, float, float, float, float, float] = field(
        default_factory=lambda: no_default(field="Velocity.val"), metadata=required
    )
    """
    A tuple of six floats. 
     - x in (m/s): Velocity of the x (forward) coordinate of the center of the cuboid.
     - y in (m/s): Velocity of the y (left) coordinate of the center of the cuboid.
     - z in (m/s): Velocity of the z (up) coordinate of the center of the cuboid.
     - rx in (rad/s): Velocity of the roll angle.
     - ry in (rad/s): Velocity of the pitch angle.
     - rz in (rad/s): Velocity of the yaw angle.
    """

    name: Literal["velocity"] = field(default="velocity")
    """Is always 'velocity'"""


@dataclass
class Velocity__UStdDev(VectorData):
    """Uncertainty for `Velocity` as standard deviation around the parent indicated value."""

    val: tuple[float, float, float, float, float, float] = field(
        default_factory=lambda: no_default(field="Velocity__UStdDev.val"), metadata=required
    )
    """Standard deviation around the parent indicated value. """

    name: Literal["velocity/ustddev"] = field(default="velocity/ustddev")
    """Is always 'velocity/ustddev'."""


@dataclass
class Acceleration(VectorData):
    """Acceleration data of the object."""

    val: tuple[float, float, float, float, float, float] = field(
        default_factory=lambda: no_default(field="Acceleration.val"), metadata=required
    )
    """
    A tuple of six floats. 
     - x in (m/s^2): Acceleration of the x coordinate of the center of the cuboid
     - y in (m/s^2): Acceleration of the y coordinate of the center of the cuboid
     - z in (m/s^2): Acceleration of the z coordinate of the center of the cuboid
     - rx in (rad/s^2): Acceleration of the roll angle.
     - ry in (rad/s^2): Acceleration of the pitch angle.
     - rz in (rad/s^2): Acceleration of the yaw angle.
    """

    name: Literal["acceleration"] = field(default="acceleration")
    """Is always 'acceleration'"""


@dataclass
class Acceleration__UStdDev(VectorData):
    """Uncertainty for `Acceleration` as standard deviation around the parent indicated value."""

    val: tuple[float, float, float, float, float, float] = field(
        default_factory=lambda: no_default(field="Acceleration__UStdDev.val"), metadata=required
    )
    """Standard deviation around the parent indicated value. """

    name: Literal["acceleration/ustddev"] = field(default="acceleration/ustddev")
    """Is always 'acceleration/ustddev'."""


@dataclass
class Classification__Uncertainties(VectorData):
    """
    Uncertainty values for the classification of the object.
    """

    val: tuple[float, float, float, float, float, float, float, float, float, float, float, float, float] = field(
        default_factory=lambda: no_default(field="Classification__Uncertainties.val"), metadata=required
    )
    """Tuple with classification probabilities (must accumulate to 1), in the following order: 
     - animal
     - vehicle/bicycle
     - vehicle/bus
     - vehicle/car
     - human/pedestrian
     - vehicle/mobility_device
     - vehicle/motorcycle
     - other
     - pushable_pullable
     - vehicle/railvehicle
     - vehicle/trailer
     - vehicle/truck
     - vehicle/van
    """

    name: Literal["classification/uncertainties"] = field(default="classification/uncertainties")
    """Is always 'classification/uncertainties'"""

    def __post_init__(self) -> None:
        if not math.isclose(sum(self.val), 1.0):
            raise ValueError(f"Uncertainty values for {self.__class__.__name__} must sum to 1.")


@dataclass
class IsRecorder(BooleanData):
    """Indicates whether the object is the recording entity in this scenario."""

    val: bool = field(default_factory=lambda: no_default(field="IsRecorder.val"), metadata=required)
    """If True, this object is the recording entity of the scenario."""

    name: Literal["is_recorder"] = field(default="is_recorder")
    """Is always 'is_recorder'."""


@dataclass
class AttachedTo(TextData):
    """An attribute for objects that are attached to other objects."""

    val: ObjectUid = field(default_factory=lambda: no_default(field="AttachedTo.val"), metadata=required)
    """The UID of the object this object is attached to."""

    name: Literal["attached_to"] = field(default="attached_to")
    """Is always 'attached_to'."""


class BestDetectedSideValue(str, Enum):
    """Possible values of `BestDetectedSide`"""

    LEFT = "left"
    """The object's left side is detected best."""

    RIGHT = "right"
    """The object's right side is detected best."""

    FRONT = "front"
    """The object's front side is detected best."""

    BACK = "back"
    """The object's back side is detected best."""

    TOP = "top"
    """The object's top side is detected best."""


@dataclass
class BestDetectedSide(TextData):
    """The classification of the best detectable bbox-side of the object."""

    val: BestDetectedSideValue = field(default_factory=lambda: no_default(field="BestDetectedSide.val"), metadata=required)
    """See `BestDetectedSideValue` for possible values."""

    name: Literal["best_detected_side"] = field(default="best_detected_side")
    """Is always 'best_detected_side'"""


@dataclass
class BestDetectedPoint(VectorData):
    """Best-detected point of the object."""

    val: Union[tuple[float, float], tuple[float, float, float]] = field(
        default_factory=lambda: no_default(field="BestDetectedPoint.val"), metadata=required
    )
    """(x, y) or (x, y, z) coordinates of best detectable point of bbox of road users."""

    name: Literal["best_detected_point"] = field(default="best_detected_point")
    """Is always 'best_detected_point'."""



"""Operator input information"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from dataclasses import dataclass, field
from typing import Literal

from apischema.metadata import required
from uai_openlabel import BooleanData, NumberData, no_default


@dataclass
class Interior__HasRider(BooleanData):
    """True iff the vehicle has a rider."""

    val: bool = field(default_factory=lambda: no_default(field="Interior__HasRider.val"), metadata=required)
    """True iff the vehicle has a rider."""

    name: Literal["interior/has_rider"] = field(default="interior/has_rider")
    """Is always 'interior/has_rider'"""


@dataclass
class Interior__SteeringAngle(NumberData):
    """Steering wheel angle (left=positive)"""

    val: float = field(default_factory=lambda: no_default(field="Interior__SteeringAngle.val"), metadata=required)
    """Float of angle in (rad)."""

    name: Literal["interior/steering_angle"] = field(default="interior/steering_angle")
    """Is always 'interior/steering_angle'"""


@dataclass
class Interior__SteeringAngle__UStdDev(NumberData):
    """Uncertainty for `Interior__SteeringAngle` as standard deviation around the parent indicated value."""

    val: float = field(default_factory=lambda: no_default(field="Interior__SteeringAngle__UStdDev.val"), metadata=required)
    """Standard deviation around the parent indicated value. """

    name: Literal["interior/steering_angle/ustddev"] = field(default="interior/steering_angle/ustddev")
    """Is always 'interior/steering_angle/ustddev'."""


@dataclass
class Interior__AutomatedControl__Longitudinal(BooleanData):
    """Boolean indicating automated control of the vehicle.

    True iff the longitudinal motion (acceleration, braking) is at this moment effectively decided / effected by the automated system.
    This includes classical cruise control, ACC, emergency braking and any more advanced SAE Level 3+ system.
    It excludes enhancing systems such as brake boosters, ABS, ESP, and advising systems such as forward collision warning.
    """

    val: bool = field(
        default_factory=lambda: no_default(field="Interior__AutomatedControl__Longitudinal.val"), metadata=required
    )
    """True iff the automated system decides the longitudinal motion at this moment."""

    name: Literal["interior/automated_control/longitudinal"] = field(default="interior/automated_control/longitudinal")
    """Is always 'interior/automated_control/longitudinal'"""


@dataclass
class Interior__AutomatedControl__Lateral(BooleanData):
    """Boolean indicating automated control of the vehicle.

    True iff the lateral motion (steering) is at this moment effectively decided / effected by the automated system.
    This includes lane keeping and lane change assistants, parking steering assistants and any more advanced SAE Level 3+ system.
    It excludes enhancing systems such as power steering or ESP, and advising systems such as lane departure warning.
    """

    val: bool = field(default_factory=lambda: no_default(field="Interior__AutomatedControl__Lateral.val"), metadata=required)
    """True iff the automated system decides the lateral motion at this moment."""

    name: Literal["interior/automated_control/longitudinal"] = field(default="interior/automated_control/longitudinal")
    """Is always 'interior/automated_control/lateral'"""


@dataclass
class Interior__BrakePedal(NumberData):
    """Position of brake pedal.

    Position of brake pedal as range with 1=pedal fully pressed and 0=unpressed.
    """

    val: float = field(default_factory=lambda: no_default(field="Interior__BrakePedal.val"), metadata=required)
    """Proportion of accelerator pedal push position"""

    name: Literal["interior/brake_pedal"] = field(default="interior/brake_pedal")
    """Is always 'interior/brake_pedal'"""

    def __post_init__(self) -> None:
        if self.val < 0.0 or self.val > 1.0:
            raise ValueError("val can only be between 0 and 1.")


@dataclass
class Interior__AcceleratorPedal(NumberData):
    """Position of accelerator pedal.

    Position of accelerator pedal as range with 1=pedal fully pressed and 0=unpressed.
    """

    val: float = field(default_factory=lambda: no_default(field="Interior__AcceleratorPedal.val"), metadata=required)
    """Proportion of accelerator pedal push position"""

    name: Literal["interior/accelerator_pedal"] = field(default="interior/accelerator_pedal")
    """Is always 'interior/accelerator_pedal'"""

    def __post_init__(self) -> None:
        if self.val < 0.0 or self.val > 1.0:
            raise ValueError("val can only be between 0 and 1.")


@dataclass
class Interior__Gear(NumberData):
    """Currently selected gear.

    Currently selected gear, where any positive number indicates the vehicle-specific gear number, and with the following special numbers:
    - 0: Neutral or clutch pedal pushed
    - -1: Reverse
    - -2: Park
    """

    val: int = field(default_factory=lambda: no_default(field="Interior__Gear.val"), metadata=required)
    """Gear value"""

    name: Literal["interior/gear"] = field(default="interior/gear")
    """Is always 'interior/gear'"""


@dataclass
class Interior__Wiper(BooleanData):
    """Boolean indicating if wiper is turned on.

    Boolean attribute that indicates whether the windshield wiper is on. This includes time intervals when the
    wiper is not in motion, but will move again after a fixed interval. It does not include an 'automatic' setting
    when, for absence of rain, the wipers will not move in the foreseeable future.
    """

    val: bool = field(default_factory=lambda: no_default(field="Interior__Wiper.val"), metadata=required)
    """True iff the wipers are on."""

    name: Literal["interior/wiper"] = field(default="interior/wiper")
    """Is always 'interior/wiper'"""



"""Traffic density and intensity"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass, field
from typing import Literal

from apischema.metadata import required
from uai_openlabel import (
    NumberData,
    NumberType,
    no_default,
)


@dataclass
class Traffic__Density(NumberData):
    """Density of the traffic on the road section the vehicle is currently on.

    The density of the traffic is the number of vehicles with the center of their bounding box within
    a 50m range before and after the vehicle containing this attribute.

    The vehicle containing this attribute is counted as well.
    """

    val: float = field(default_factory=lambda: no_default(field="Traffic__Density.val"), metadata=required)
    """Number of vehicles """

    name: Literal["traffic/density"] = field(default="traffic/density")
    """Is always 'traffic/density'"""

    type: Literal[NumberType.Value] = field(default=NumberType.Value)
    """Is always 'value'"""


@dataclass
class Traffic__Density__UStdDev(NumberData):
    """Uncertainty for `Traffic__Density` as standard deviation around the parent indicated value."""

    val: float = field(default_factory=lambda: no_default(field="Traffic__Density__UStdDev.val"), metadata=required)
    """Standard deviation around the parent indicated value. """

    name: Literal["traffic/density/ustddev"] = field(default="traffic/density/ustddev")
    """Is always 'traffic/density/ustddev'. """


@dataclass
class Traffic__Volume(NumberData):
    """Intensity of the traffic on the road section the vehicle is currently on.

    The intensity of the traffic is measured over a time frame of at least 10 minutes.
    The unit of the traffic intensity is vehicles per hour.

    The vehicle containing this attribute is counted as well.
    """

    val: float = field(default_factory=lambda: no_default(field="Traffic__Volume.val"), metadata=required)
    """The traffic intensity in vehicles per hour"""

    name: Literal["traffic/volume"] = field(default="traffic/volume")
    """Is always 'traffic/volume'"""

    type: Literal[NumberType.Value] = field(default=NumberType.Value)
    """Is always 'value'"""


@dataclass
class Traffic__Volume__UStdDev(NumberData):
    """Uncertainty for `Traffic__Volume` as standard deviation around the parent indicated value."""

    val: float = field(default_factory=lambda: no_default(field="Traffic__Volume__UStdDev.val"), metadata=required)
    """Standard deviation around the parent indicated value. """

    name: Literal["traffic/volume/ustddev"] = field(default="traffic/volume/ustddev")
    """Is always 'traffic/volume/ustddev'. """



"""Classes associated with an object of classification 'vehicle/railvehicle'"""

# Copyright © 2024 understandAI GmbH
#
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files
# (the “Software”), to deal in the Software without restriction, including without limitation the rights to use, copy, modify,
# merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED “AS IS”, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
# CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.


from dataclasses import dataclass, field
from typing import Literal

from apischema.metadata import required
from uai_openlabel import Object as BaseObject
from uai_openlabel import ObjectInFrame as BaseObjectInFrame
from uai_openlabel import no_default

from aveas_openlabel.object_data.unsteerable import ObjectData__Unsteerable
from aveas_openlabel.object_in_frame_data.unsteerable import (
    ObjectInFrameData__Unsteerable,
)


@dataclass
class RailVehicleInFrame(BaseObjectInFrame):
    """
    This object contains the dynamic attributes of objects with the classification 'vehicle/railvehicle'.

    For the static attributes, see `RailVehicle`.
    Instances of this class are found in `Frame` instances inside `AveasOpenLabel.frames`.
    """

    object_data: ObjectInFrameData__Unsteerable = field(
        default_factory=lambda: no_default(field="RailVehicleInFrame.object_data"), metadata=required
    )
    """Contains the object's dynamic, time-dependent attributes"""


@dataclass
class RailVehicle(BaseObject):
    """
    This object contains the static attributes of objects with the classification 'vehicle/railvehicle'.

    For the dynamic attributes, see `RailVehicleInFrame`.
    Instances of this class are found in `AveasOpenLabel.objects`.
    """

    name: str = field(default_factory=lambda: no_default(field="RailVehicle.name"), metadata=required)
    """A friendly identifier of the element, is not unique but employed by human users to rapidly identify elements in the scene."""

    type: Literal["vehicle/railvehicle"] = field(default="vehicle/railvehicle")
    """The classification of the object"""

    object_data: ObjectData__Unsteerable = field(
        default_factory=lambda: no_default(field="RailVehicle.object_data"), metadata=required
    )
    """Contains the object's static attributes"""
